#* 
#* ------------------------------------------------------------------
#* Dialogs.tcl - Dialog Boxes
#* Created by Robert Heller on Sun Jan 28 21:34:32 2007
#* ------------------------------------------------------------------
#* Modification History: $Log: headerfile.text,v $
#* Modification History: Revision 1.1  2002/07/28 14:03:50  heller
#* Modification History: Add it copyright notice headers
#* Modification History:
#* ------------------------------------------------------------------
#* Contents:
#* ------------------------------------------------------------------
#*  
#*     Generic Project
#*     Copyright (C) 2005  Robert Heller D/B/A Deepwoods Software
#* 			51 Locke Hill Road
#* 			Wendell, MA 01379-9728
#* 
#*     This program is free software; you can redistribute it and/or modify
#*     it under the terms of the GNU General Public License as published by
#*     the Free Software Foundation; either version 2 of the License, or
#*     (at your option) any later version.
#* 
#*     This program is distributed in the hope that it will be useful,
#*     but WITHOUT ANY WARRANTY; without even the implied warranty of
#*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*     GNU General Public License for more details.
#* 
#*     You should have received a copy of the GNU General Public License
#*     along with this program; if not, write to the Free Software
#*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#* 
#*  
#* 

package require Tk
package require BWidget
package require snit

#***********************************************
# Readonly text bindings -- same as Text bindings, except with out any bindings
# for inserting or modifying the contents (all of the 'destructive' binding
# have been removed). (Copied from text.tcl.)
#***********************************************

catch {
# Standard Motif bindings:

bind ROText <1> {
    tk::TextButton1 %W %x %y
    %W tag remove sel 0.0 end
}
bind ROText <B1-Motion> {
    set tk::Priv(x) %x
    set tk::Priv(y) %y
    tk::TextSelectTo %W %x %y
}
bind ROText <Double-1> {
    set tk::Priv(selectMode) word
    tk::TextSelectTo %W %x %y
    catch {%W mark set insert sel.last}
    catch {%W mark set anchor sel.first}
}
bind ROText <Triple-1> {
    set tk::Priv(selectMode) line
    tk::TextSelectTo %W %x %y
    catch {%W mark set insert sel.last}
    catch {%W mark set anchor sel.first}
}
bind ROText <Shift-1> {
    tk::TextResetAnchor %W @%x,%y
    set tk::Priv(selectMode) char
    tk::TextSelectTo %W %x %y
}
bind ROText <Double-Shift-1>	{
    set tk::Priv(selectMode) word
    tk::TextSelectTo %W %x %y 1
}
bind ROText <Triple-Shift-1>	{
    set tk::Priv(selectMode) line
    tk::TextSelectTo %W %x %y
}
bind ROText <B1-Leave> {
    set tk::Priv(x) %x
    set tk::Priv(y) %y
    tk::TextAutoScan %W
}
bind ROText <B1-Enter> {
    tk::CancelRepeat
}
bind ROText <ButtonRelease-1> {
    tk::CancelRepeat
}
bind ROText <Control-1> {
    %W mark set insert @%x,%y
}
bind ROText <Left> {
    tk::TextSetCursor %W insert-1c
}
bind ROText <Right> {
    tk::TextSetCursor %W insert+1c
}
bind ROText <Up> {
    tk::TextSetCursor %W [tk::TextUpDownLine %W -1]
}
bind ROText <Down> {
    tk::TextSetCursor %W [tk::TextUpDownLine %W 1]
}
bind ROText <Shift-Left> {
    tk::TextKeySelect %W [%W index {insert - 1c}]
}
bind ROText <Shift-Right> {
    tk::TextKeySelect %W [%W index {insert + 1c}]
}
bind ROText <Shift-Up> {
    tk::TextKeySelect %W [tk::TextUpDownLine %W -1]
}
bind ROText <Shift-Down> {
    tk::TextKeySelect %W [tk::TextUpDownLine %W 1]
}
bind ROText <Control-Left> {
    tk::TextSetCursor %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord]
}
bind ROText <Control-Right> {
    tk::TextSetCursor %W [tk::TextNextWord %W insert]
}
bind ROText <Control-Up> {
    tk::TextSetCursor %W [tk::TextPrevPara %W insert]
}
bind ROText <Control-Down> {
    tk::TextSetCursor %W [tk::TextNextPara %W insert]
}
bind ROText <Shift-Control-Left> {
    tk::TextKeySelect %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord]
}
bind ROText <Shift-Control-Right> {
    tk::TextKeySelect %W [tk::TextNextWord %W insert]
}
bind ROText <Shift-Control-Up> {
    tk::TextKeySelect %W [tk::TextPrevPara %W insert]
}
bind ROText <Shift-Control-Down> {
    tk::TextKeySelect %W [tk::TextNextPara %W insert]
}
bind ROText <Prior> {
    tk::TextSetCursor %W [tk::TextScrollPages %W -1]
}
bind ROText <Shift-Prior> {
    tk::TextKeySelect %W [tk::TextScrollPages %W -1]
}
bind ROText <Next> {
    tk::TextSetCursor %W [tk::TextScrollPages %W 1]
}
bind ROText <Shift-Next> {
    tk::TextKeySelect %W [tk::TextScrollPages %W 1]
}
bind ROText <Control-Prior> {
    %W xview scroll -1 page
}
bind ROText <Control-Next> {
    %W xview scroll 1 page
}

bind ROText <Home> {
    tk::TextSetCursor %W {insert linestart}
}
bind ROText <Shift-Home> {
    tk::TextKeySelect %W {insert linestart}
}
bind ROText <End> {
    tk::TextSetCursor %W {insert lineend}
}
bind ROText <Shift-End> {
    tk::TextKeySelect %W {insert lineend}
}
bind ROText <Control-Home> {
    tk::TextSetCursor %W 1.0
}
bind ROText <Control-Shift-Home> {
    tk::TextKeySelect %W 1.0
}
bind ROText <Control-End> {
    tk::TextSetCursor %W {end - 1 char}
}
bind ROText <Control-Shift-End> {
    tk::TextKeySelect %W {end - 1 char}
}

bind ROText <Control-Tab> {
    focus [tk_focusNext %W]
}
bind ROText <Control-Shift-Tab> {
    focus [tk_focusPrev %W]
}
bind ROText <Select> {
    %W mark set anchor insert
}
bind ROText <<Copy>> {
    tk_textCopy %W
}
# Additional emacs-like bindings:

bind ROText <Control-a> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W {insert linestart}
    }
}
bind ROText <Control-b> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W insert-1c
    }
}
bind ROText <Control-e> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W {insert lineend}
    }
}
bind ROText <Control-f> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W insert+1c
    }
}
bind ROText <Control-n> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W [tk::TextUpDownLine %W 1]
    }
}
bind ROText <Control-p> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W [tk::TextUpDownLine %W -1]
    }
}
if {[string compare $tcl_platform(platform) "windows"]} {
bind ROText <Control-v> {
    if {!$tk_strictMotif} {
	tk::TextScrollPages %W 1
    }
}
}

bind ROText <Meta-b> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord]
    }
}
bind ROText <Meta-f> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W [tk::TextNextWord %W insert]
    }
}
bind ROText <Meta-less> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W 1.0
    }
}
bind ROText <Meta-greater> {
    if {!$tk_strictMotif} {
	tk::TextSetCursor %W end-1c
    }
}
# Macintosh only bindings:

# if text black & highlight black -> text white, other text the same
if {[string equal $tcl_platform(platform) "macintosh"]} {
bind ROText <FocusIn> {
    %W tag configure sel -borderwidth 0
    %W configure -selectbackground systemHighlight -selectforeground systemHighlightText
}
bind ROText <FocusOut> {
    %W tag configure sel -borderwidth 1
    %W configure -selectbackground white -selectforeground black
}
bind ROText <Option-Left> {
    tk::TextSetCursor %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord]
}
bind ROText <Option-Right> {
    tk::TextSetCursor %W [tk::TextNextWord %W insert]
}
bind ROText <Option-Up> {
    tk::TextSetCursor %W [tk::TextPrevPara %W insert]
}
bind ROText <Option-Down> {
    tk::TextSetCursor %W [tk::TextNextPara %W insert]
}
bind ROText <Shift-Option-Left> {
    tk::TextKeySelect %W [tk::TextPrevPos %W insert tcl_startOfPreviousWord]
}
bind ROText <Shift-Option-Right> {
    tk::TextKeySelect %W [tk::TextNextWord %W insert]
}
bind ROText <Shift-Option-Up> {
    tk::TextKeySelect %W [tk::TextPrevPara %W insert]
}
bind ROText <Shift-Option-Down> {
    tk::TextKeySelect %W [tk::TextNextPara %W insert]
}

# End of Mac only bindings
}

}

namespace eval Dialogs {
  #**********************************************
  # String edit popup dialog
  #**********************************************
  snit::type EditStringDialog {
    # ensemble command pragmas:
    pragma -hastypeinfo    no
    pragma -hastypedestroy no
    pragma -hasinstances   no

    # Type Components:
    typecomponent dialog		;# Dialog box itself
    typecomponent  title		;# Name of what we are editing
    typecomponent  entry		;# String we are editing

    #**************************************
    # Type constructor
    #**************************************
    typeconstructor {
      set dialog [Dialog::create .editStringDialog \
			-transient yes -parent . -default 0 -cancel 1 \
			-title {Edit String} -modal local -bitmap question \
			-side bottom]
      $dialog add -name ok -text OK -command [mytypemethod _OK]
      $dialog add -name cancel -text Cancel -command [mytypemethod _Cancel]
      $dialog add -name help -text Help -command "BWHelp::HelpTopic EditStringDialog"
      set frame [$dialog getframe]
      set title [Label::create $frame.title -text {Edit String} -anchor c]
      pack $title -fill x -anchor c
      set entry [Entry::create $frame.entry -text {}]
      pack $entry -fill x
    }
    typemethod _OK {} {
      $dialog withdraw
      return [$dialog enddialog ok]
    }
    typemethod _Cancel {} {
      $dialog withdraw
      return [$dialog enddialog cancel]
    }
    typemethod draw {args} {
      $entry configure -text "[from args -currentstring {}]"
      $title configure -text "[from args -title {Edit String}]"
      $dialog configure -title "[$title cget -text]"
      $dialog configure -parent [from args -parent .]
      wm transient [winfo toplevel $dialog] [$dialog cget -parent]
      set button [$dialog draw]
      return [list $button "[$entry cget -text]"]
    }
  }
  #**********************************************
  # Subprocess Dialog (subprocess log)
  #**********************************************
  snit::widget SubprocessDialog {
    # Hull type -- toplevel
    hulltype toplevel
    # Widget class
    widgetclass SubprocessDialog
    # Components:
    component mainFrame 		;# Main frame
    component logScroll			;# Log scroll window
    component logText			;# Log window
    component dismis			;# Dismis button
    # Instance variables:
    variable status {}			;# Status
    variable _Pipe {}			;# I/O pipe from the process's stdout
    # Options:
    #					Title:
    option -title -default {Backgroud Shell Process} -configuremethod _SetTitle
    method _SetTitle {option value} {
      wm title $win "$value"
      set options($option) "$value"
    }
    #					Parent:
    option -parent -default . -configuremethod _SetTransient
    method _SetTransient {option value} {
      wm transient $win $value
      set options($option) $value
    }
    option -command -default {}	-readonly yes ;# Command to execute
    # Delegated options:
    delegate option -menu to hull	;# Delegate -menu to hull
    delegate option -width to logText	;# Delegate -width to the log textarea
    delegate option -height to logText  ;# Delegate -height to the log textarea

    #**********************************************
    # Constructor:
    #**********************************************
    constructor {args} {
#      puts stderr "*** $type create $self $args"
      # Define min and max window sizes
      wm maxsize $win 1024 768
      wm minsize $win 10 10
      # Capture control of the close function of the system menu
      wm protocol $win WM_DELETE_WINDOW [mymethod _Close]
      # Install the main frame
      install mainFrame using MainFrame $win.mainFrame\
	 -textvariable [myvar status]
#      puts stderr "*** $self constructor: mainFrame = $mainFrame"
      pack $mainFrame -expand yes -fill both
      $mainFrame showstatusbar status
      # Install the log scrollwindow      
      install logScroll using ScrolledWindow [$mainFrame getframe].logScroll \
			-scrollbar both -auto both
#      puts stderr "*** $self constructor: logScroll = $logScroll"
      pack $logScroll -expand yes -fill both
      # Install the abort/dismis button
      install dismis using Button [$mainFrame getframe].dismis \
			-text "Abort" -state normal \
			-command [mymethod _Close]
#      puts stderr "*** $self constructor: dismis = $dismis"
      pack $dismis -fill x
      # Install the log text area.
      install logText using text [$logScroll getframe].text -wrap none
#      puts stderr "*** $self constructor: logText = $logText"
      # Make the log area 'read-only'
      bindtags $logText [list $logText ROText . all]
      pack $logText -expand yes -fill both
      $logScroll setwidget $logText
      $self configurelist $args
      # Start the process.
      if {[string equal "$options(-command)" {}]} {
	error "Required option -command is missing!"
      }
      if {[catch {open "|$options(-command)" r} _Pipe]} {
	tk_messageBox -type ok -icon error -message "Could not start process: $_Pipe"
	set _Pipe {}
        return
      }
      fileevent $_Pipe readable [mymethod _PipeToLog]
    }
    #**********************************************
    # Method to Abort process or close window
    #**********************************************
    method _Close {} {
      if {[string equal "$_Pipe" {}]} {
	destroy $self
      } else {
	if {[string equal \
		[tk_messageBox -type yesno -icon question -parent $win \
		     -message "Are you sure you want to abort this process?"] \
		no]} {return}
	catch {close $_Pipe} error
        set _Pipe {}
        $logText insert end "$error\n"
        $logText see end-1c
	$dismis configure -text "Dismis"
      }
    }
    #**********************************************
    # Method to collect stdout/stderr from process
    #**********************************************
    method _PipeToLog {} {
      if {[gets $_Pipe line] >= 0} {
	$logText insert end "$line\n"
      } else {
	catch {close $_Pipe} error
	set _Pipe {}
        $dismis configure -text "Dismis"
      }
      $logText see end-1c
    }
    #**********************************************
    # Type method to create a new process window
    #**********************************************
    typemethod draw {args} {
      return [eval [list $type create .subprocessDialog%AUTO%] $args]
    }
  }
  #**********************************************
  # Dialog to select file to print
  #**********************************************
  snit::type SelectFileToPrintDialog {
    # ensemble command pragmas:
    pragma -hastypeinfo    no
    pragma -hastypedestroy no
    pragma -hasinstances   no

    # Type Components:
    typecomponent dialog		;# Dialog box itself
    typecomponent   rbLF		;# Radiobox label frame
    typecomponent     sourceRB		;# Source radiobutton
    typecomponent     sourceE		;# Source entry (noteditable)
    typecomponent     outputRB		;# Output radiobutton
    typecomponent     outputE		;# Output entry (noteditable)
    # Type variables:
    typevariable _WhichFile Output	;# Which file to print
    #**************************************
    # Type constructor -- create dialog box.
    #**************************************
    typeconstructor {
      set dialog [Dialog::create .selectFileToPrintDialog \
			-transient yes -parent . -default 0 -cancel 1 \
			-title {Select File to print} -modal local \
			-bitmap question -side bottom]
      $dialog add -name print -text Print -command [mytypemethod _Print]
      $dialog add -name cancel -text Cancel -command [mytypemethod _Cancel]
      $dialog add -name help -text Help -command "BWHelp::HelpTopic SelectFileToPrintDialog"
      set frame [$dialog getframe]
      set rbLF [LabelFrame::create $frame.rbLF -text "Select File:" -side top]
      pack $rbLF -expand yes -fill both
      set frame1 [$rbLF getframe]
      set sourceRB [radiobutton $frame1.sourceRB \
				-text "Source:" \
				-variable [mytypevar _WhichFile] \
				-value Source]
      grid $sourceRB -column 0 -row 0 -sticky nws
      set sourceE [Entry::create $frame1.sourceE -editable no]
      grid $sourceE -column 1 -row 0 -sticky news
      set outputRB [radiobutton $frame1.outputRB \
				-text "Output:" \
				-variable [mytypevar _WhichFile] \
				-value Output]
      grid $outputRB -column 0 -row 1 -sticky nws
      set outputE [Entry::create $frame1.outputE -editable no]
      grid $outputE -column 1 -row 1 -sticky news
      grid columnconfigure $frame1 0 -weight 0 -uniform yes
      grid columnconfigure $frame1 1 -weight 1 -uniform yes

    }
    #**************************************
    # Typemethod bound to Print button -- return selected file
    #**************************************
    typemethod _Print {} {
      $dialog withdraw
      switch $_WhichFile {
 	Source {
	  return [$dialog enddialog "[$sourceE cget -text]"]
	}
 	Output {
	  return [$dialog enddialog "[$outputE cget -text]"]
	}
      }
    }
    #**************************************
    # Typemethod bound to Cancel button -- return the null string
    #**************************************
    typemethod _Cancel {} {
      $dialog withdraw
      return [$dialog enddialog {}]
    }
    #**************************************
    # Typemethod to fire up the dialog
    #**************************************
    typemethod draw {args} {
      set source [from args -source {}]
      set output [from args -output {}]
      set parent [from args -parent .]
      $dialog configure -parent $parent
      wm transient [winfo toplevel $dialog] [$dialog cget -parent]
      $sourceE configure -text "$source"
      $outputE configure -text "$output"
      return [$dialog draw]
    }
  }
}


package provide Dialogs 1.0
