#* 
#* ------------------------------------------------------------------
#* OpenTaxSolver.tcl - Open Tax Solver GUI, in Tcl/Tk
#* Created by Robert Heller on Sat Jan 27 12:18:01 2007
#* ------------------------------------------------------------------
#* Modification History: $Log: headerfile.text,v $
#* Modification History: Revision 1.1  2002/07/28 14:03:50  heller
#* Modification History: Add it copyright notice headers
#* Modification History:
#* ------------------------------------------------------------------
#* Contents:
#* ------------------------------------------------------------------
#*  
#*     Generic Project
#*     Copyright (C) 2005  Robert Heller D/B/A Deepwoods Software
#* 			51 Locke Hill Road
#* 			Wendell, MA 01379-9728
#* 
#*     This program is free software; you can redistribute it and/or modify
#*     it under the terms of the GNU General Public License as published by
#*     the Free Software Foundation; either version 2 of the License, or
#*     (at your option) any later version.
#* 
#*     This program is distributed in the hope that it will be useful,
#*     but WITHOUT ANY WARRANTY; without even the implied warranty of
#*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*     GNU General Public License for more details.
#* 
#*     You should have received a copy of the GNU General Public License
#*     along with this program; if not, write to the Free Software
#*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#* 
#*  
#* 

package require Tk

package require BWHelp
package require BWStdMenuBar
package require TaxForms

namespace eval OpenTaxSolver {}

global HelpDir
set HelpDir [file join [file dirname [file dirname [file dirname \
                                                        [info script]]]] Help]
global ImageDir
set ImageDir [file join [file dirname [file dirname [info script]]] Common]


namespace eval OpenTaxSolver {

  variable INVOCATION_PATH [info nameofexecutable]
  variable BINARYPATH      [file dirname $INVOCATION_PATH]
  variable BASEDIRPATH     [file join [file dirname $BINARYPATH] examples_and_templates]
#  puts "*** INVOCATION_PATH = $INVOCATION_PATH"
#  puts "*** BINARYPATH = $BINARYPATH"
#  puts "*** BASEDIRPATH = $BASEDIRPATH"
  image create photo OpenTaxSolverLogo -file [file join $::ImageDir otslogo.ppm]

  # Window manager configurations
  wm positionfrom . user
  wm sizefrom . ""
  wm maxsize . 1265 994
  wm minsize . 1 1
  wm protocol . WM_DELETE_WINDOW {CareFulExit}
  wm title . {Open Tax Solver}

# Menu bar
  set menubar [StdMenuBar::MakeMenu \
	-file {"&File" {file} {file} 0 {
		{command "&New" {file:new} "" {} -state disabled}
		{command "&Open..." {file:open} "Open and Load a form" {Ctrl o}
				-command OpenTaxSolver::OpenForm}
	        {command "&Save"    {file:save} "Save Form Data" {Ctrl s}
				-command OpenTaxSolver::SaveForm}
	        {command "Save &As..."    {file:saveas} "Save Form Data" {Ctrl a}
				-command OpenTaxSolver::SaveFormAs}
		{command "&Compute Tax" {file:run} "Compute Tax" {Ctrl c} 
				-command OpenTaxSolver::ComputeTax}
		{command "&Print..." {file:print} "Print Taxes" {Ctrl p}
				-command OpenTaxSolver::PrintTaxes}
		{command "&Close" {file:close} "Close the application" {Ctrl q}
				-command OpenTaxSolver::CareFulExit}
		{command "E&xit" {file:exit} "Close the application" {Ctrl q}
				-command OpenTaxSolver::CareFulExit}
		}
	} \
	-options {"&Options" {options} {options} 0 {
		{command "Change Form &Title" {options:change} \
				"Change Form Title" {Ctrl t} \
				-command OpenTaxSolver::ChangeFormTitle}
		}
	}]
# Main GUI window
  variable Status
  variable Main [MainFrame::create .main -menu $menubar \
				-textvariable OpenTaxSolver::Status]
  pack $Main -expand yes -fill both
  $Main showstatusbar status

  set frame [$Main getframe]
# Solver notebook (tabbed panel)
  variable SolverNotebook [NoteBook::create $frame.notebook -arcradius 5 -tabbevelsize 5]
  pack $SolverNotebook -expand yes -fill both -padx 5 -pady 5

# Main tab -- select a solver program and load a form
  set MainSolverTab [$SolverNotebook insert end mainSolver -text {Open-Tax-Solver}]

# Logo
  pack [Label::create $MainSolverTab.logo -image OpenTaxSolverLogo \
		-relief raised -borderwidth 5] -anchor c -padx 5 -pady 5


# Radio box frame
  variable SelectProg [LabelFrame::create $MainSolverTab.selectprog \
			-text {Select Tax Program:} -side top \
			-font {Helvectia -15}]
  pack $SelectProg -expand yes -fill both -padx 10 -pady 10
  set selframe [$SelectProg getframe]

# Known list of forms and solver programs
  variable FormList
  array set FormList {
    0,0 {{US 1040 (w/Scheds A,B,D)} normal taxsolve_US_1040_2007 US_1040}
    1,0 {{US 1040 Sched C} normal taxsolve_US_1040_Sched_C_2007 US_1040_Sched_C}
    2,0 {{CA State 540} normal taxsolve_CA_540_2007 CA_540}
    3,0 {{MA State 1} normal taxsolve_MA_1_2007 MA_1}
    4,0 {{NC State DC400}  normal taxsolve_NC_D400_2007 NC_400}
    5,0 {{NJ State 1040} disabled taxsolve_NJ_1040_2007 NJ_1040}
    0,1 {{NY State IT201} disabled taxsolve_NY_IT201_2007 NY_IT201}
    1,1 {{OH State IT1040} disabled taxsolve_OH_IT1040_2007 OH_IT1040}
    2,1 {{PA STATE 40} disabled taxsolve_PA_40_2007 PA_40}
    3,1 {{VA State 760} disabled taxsolve_VA_760_2007 VA_760}
    4,1 {{Other} normal Other {}}
  }

  variable FormDirectories

# Select default value (US 1040)
  variable SelectedProgram [lindex $FormList(0,0) 2]

# Create the gridded array of radio buttons
  foreach rb [array names FormList] {
    foreach {r c} [split $rb {,}] {break}
    foreach {label state value directory} $FormList($rb) {break}
    set FormDirectories($value) "$directory"
    grid [radiobutton $selframe.rb${c}${r} \
			-text "$label" \
			-variable OpenTaxSolver::SelectedProgram \
			-value "$value" \
			-state $state \
			-indicatoron yes] \
	 -column $c -row $r -sticky nws
  }
  grid columnconfigure $selframe 0 -uniform
  grid columnconfigure $selframe 1 -uniform

# Big button to open a form file
  pack [Button::create $MainSolverTab.open \
		-font {Helvectic -24 bold} \
		-text {Open a Tax File (or Template)} \
		-command OpenTaxSolver::OpenForm \
		-borderwidth 5] -anchor c


# Action buttons along the bottom
  set buttons [ButtonBox::create $frame.buttons -orient horizontal]
  pack $buttons -fill x -padx 5 -pady 5
  $buttons add -name save -text Save -font {Helvectia -18 bold} \
			-command OpenTaxSolver::SaveForm
  $buttons add -name saveas -text {Save As...} -font {Helvectia -18 bold} \
			-command OpenTaxSolver::SaveFormAs
  $buttons add -name compute -text {Compute Tax} -font {Helvectia -18 bold} \
			-command OpenTaxSolver::ComputeTax
  $buttons add -name print -text Print -font {Helvectia -18 bold} \
			-command OpenTaxSolver::PrintTaxes
  $buttons add -name exit -text Exit -font {Helvectia -18 bold} \
			-command OpenTaxSolver::CareFulExit

  $SolverNotebook raise mainSolver

# File types.
  variable FormFileTypes {
	{{Data Files} {.dat} TEXT}
	{{All Files}  *      TEXT}
  }
  variable OutputFileTypes {
	{{Output Files} {.out} TEXT}
	{{All Files}    *      TEXT}
  }
}

#*****************************************
# Action routines
#*****************************************

# Exit handler.
proc OpenTaxSolver::CareFulExit {} {
  if {[string compare \
        [tk_messageBox -default no -icon question -message {Really Quit?} \
                -title {Careful Exit} -type yesno] {yes}] == 0} {
    exit
  }
}

#**********************************
# Open a form file.
# Load a file into a fresh tab
#**********************************
proc OpenTaxSolver::OpenForm {} {
  variable FormFileTypes
  variable FormDirectories
  variable SelectedProgram
  variable BINARYPATH
  variable BASEDIRPATH
  variable SolverNotebook

  set initialdir "[file join $BASEDIRPATH $FormDirectories($SelectedProgram)]"
#  puts stderr "*** OpenTaxSolver::OpenForm: initialdir = $initialdir"

  set filename \
	[tk_getOpenFile \
		-defaultextension .dat -filetypes $FormFileTypes \
		-initialdir  "$initialdir" \
		-parent . \
		-title "Data or template to load"]
  if {[string equal "$filename" {}]} {return}
  set currentForm [TaxForms::TaxForm createformfromfile \
			"$filename" $SolverNotebook \
			[file join "$BINARYPATH" $SelectedProgram]]
  $currentForm raiseme
  $currentForm compute_size
  $SolverNotebook compute_size
}

#********************************
# Save a form to disk
#********************************
proc OpenTaxSolver::SaveForm {} {
  variable SolverNotebook
  variable FormFileTypes
  variable FormDirectories
  variable SelectedProgram
  variable BASEDIRPATH

  set page [$SolverNotebook raise]
  set currentForm [TaxForms::TaxForm findform $page]
  if {[string equal "$currentForm" {}]} {return}

  set existingfilename "[$currentForm cget -filename]"
  OpenTaxSolver::SaveFormAs "$existingfilename"
}

proc OpenTaxSolver::SaveFormAs {{filename {}}} {
  variable SolverNotebook
  variable FormFileTypes
  variable FormDirectories
  variable SelectedProgram
  variable BASEDIRPATH

  set page [$SolverNotebook raise]
  set currentForm [TaxForms::TaxForm findform $page]
  if {[string equal "$currentForm" {}]} {return}

  if {[string equal "$filename" {}]} {
    set existingfilename "[$currentForm cget -filename]"
    set existingdir "[file dirname $existingfilename]"
    set filename \
	[tk_getSaveFile \
		-defaultextension .dat -filetypes $FormFileTypes \
			-initialdir "$existingdir" \
			-initialfile "$existingfilename" \
			-parent . \
			-title "File to save form in"]
    if {[string equal "$filename" {}]} {return}
  }
  $currentForm saveme "$filename"
}

#***********************************
# Compute tax (solve form)
#***********************************
proc OpenTaxSolver::ComputeTax {} {
  variable SolverNotebook

  set page [$SolverNotebook raise]
  set currentForm [TaxForms::TaxForm findform $page]
  if {[string equal "$currentForm" {}]} {return}
  $currentForm computemytaxes
}

#***********************************
# Print solution
#***********************************
proc OpenTaxSolver::PrintTaxes  {} {
  variable SolverNotebook

  set page [$SolverNotebook raise]
  set currentForm [TaxForms::TaxForm findform $page]
  if {[string equal "$currentForm" {}]} {return}
  $currentForm printtaxes
}

#***********************************
# Change form title
#***********************************
proc OpenTaxSolver::ChangeFormTitle {} {
  variable SolverNotebook

  set page [$SolverNotebook raise]
  set currentForm [TaxForms::TaxForm findform $page]
  if {[string equal "$currentForm" {}]} {return}
  set existingtitle "[$currentForm gettitle]"
  foreach {button result} [Dialogs::EditStringDialog draw \
				-currentstring "$existingtitle" \
				-title "Edit Form Title" \
				-parent $SolverNotebook] {break}
  switch $button {
    ok {
      $currentForm settitle "$result"
    }
  }
}
